use super::{ApiError, DiscordErrorResponse};
use gejdr_core::models::accounts::RemoteUser;

static DISCORD_URL: &str = "https://discord.com/api/v10/";

pub async fn get_user_profile(token: &str) -> Result<RemoteUser, ApiError> {
    let client = reqwest::Client::new();
    let mut headers = reqwest::header::HeaderMap::new();
    headers.insert(
        reqwest::header::AUTHORIZATION,
        format!("Bearer {token}").parse().unwrap(),
    );
    let response = client
        .get(format!("{DISCORD_URL}/users/@me"))
        .headers(headers)
        .send()
        .await;
    match response {
        Ok(resp) => {
            if resp.status().is_success() {
                resp.json::<RemoteUser>()
                    .await
                    .map_err(std::convert::Into::into)
            } else {
                let error_response = resp.json::<DiscordErrorResponse>().await;
                match error_response {
                    Ok(val) => Err(ApiError::Api(val)),
                    Err(e) => Err(ApiError::Reqwest(e)),
                }
            }
        }
        Err(e) => Err(ApiError::Reqwest(e)),
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[tokio::test]
    async fn user_profile_invalid_token_results_in_401() {
        let res = get_user_profile("invalid").await;
        assert!(res.is_err());
        let err = res.err().unwrap();
        println!("Error: {err:?}");
        let expected = DiscordErrorResponse {
            code: 0,
            message: "401: Unauthorized".into(),
        };
        assert!(matches!(ApiError::Api(expected), _err));
    }

    // TODO: Find a way to mock calls to discord.com API with a
    // successful reply
}
