use sqlx::ConnectOptions;

#[derive(Debug, serde::Deserialize, Clone, Default)]
pub struct Database {
    pub host: String,
    pub port: u16,
    pub name: String,
    pub user: String,
    pub password: String,
    pub require_ssl: bool,
}

impl Database {
    #[must_use]
    pub fn get_connect_options(&self) -> sqlx::postgres::PgConnectOptions {
        let ssl_mode = if self.require_ssl {
            sqlx::postgres::PgSslMode::Require
        } else {
            sqlx::postgres::PgSslMode::Prefer
        };
        sqlx::postgres::PgConnectOptions::new()
            .host(&self.host)
            .username(&self.user)
            .password(&self.password)
            .port(self.port)
            .ssl_mode(ssl_mode)
            .database(&self.name)
            .log_statements(tracing::log::LevelFilter::Trace)
    }

    #[must_use]
    pub fn get_connection_pool(&self) -> sqlx::postgres::PgPool {
        tracing::event!(
            target: "startup",
            tracing::Level::INFO,
            "connecting to database with configuration {:?}",
            self.clone()
        );
        sqlx::postgres::PgPoolOptions::new()
            .acquire_timeout(std::time::Duration::from_secs(2))
            .connect_lazy_with(self.get_connect_options())
    }

    pub async fn migrate(pool: &sqlx::PgPool) {
        sqlx::migrate!()
            .run(pool)
            .await
            .expect("Failed to migrate the database");
    }
}
